@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 """
 Example of serving an Application using the `aiohttp.web` CLI.
 
@@ -12,13 +13,15 @@ needs (i.e. ``-H``, ``-P`` & ``entry-func``) and passes on any additional
 arguments to the `cli_app:init` function for processing.
 """
 
-from argparse import ArgumentParser
+from argparse import ArgumentParser, Namespace
 
 from aiohttp import web
 
+args_key = web.AppKey("args_key", Namespace)
 
-def display_message(req):
-    args = req.app["args"]
+
+async def display_message(req: web.Request) -> web.StreamResponse:
+    args = req.app[args_key]
     text = "\n".join([args.message] * args.repeat)
     return web.Response(text=text)
 
@@ -29,27 +32,22 @@ def init(argv):
     )
 
     # Positional argument
-    arg_parser.add_argument(
-        "message",
-        help="message to print"
-    )
+    arg_parser.add_argument("message", help="message to print")
 
     # Optional argument
     arg_parser.add_argument(
-        "--repeat",
-        help="number of times to repeat message", type=int, default="1"
+        "--repeat", help="number of times to repeat message", type=int, default="1"
     )
 
     # Avoid conflict with -h from `aiohttp.web` CLI parser
     arg_parser.add_argument(
-        "--app-help",
-        help="show this message and exit", action="help"
+        "--app-help", help="show this message and exit", action="help"
     )
 
     args = arg_parser.parse_args(argv)
 
     app = web.Application()
-    app["args"] = args
-    app.router.add_get('/', display_message)
+    app[args_key] = args
+    app.router.add_get("/", display_message)
 
     return app
