@@ -4,7 +4,6 @@ from io import StringIO
 from unittest import mock
 
 import pytest
-from async_generator import async_generator
 
 from aiohttp import payload, streams
 
@@ -18,6 +17,9 @@ def registry():
 
 
 class Payload(payload.Payload):
+    def decode(self, encoding: str = "utf-8", errors: str = "strict") -> str:
+        assert False
+
     async def write(self, writer):
         pass
 
@@ -99,18 +101,18 @@ def test_string_io_payload() -> None:
 
 
 def test_async_iterable_payload_default_content_type() -> None:
-    @async_generator
     async def gen():
-        pass
+        return
+        yield
 
     p = payload.AsyncIterablePayload(gen())
     assert p.content_type == "application/octet-stream"
 
 
 def test_async_iterable_payload_explicit_content_type() -> None:
-    @async_generator
     async def gen():
-        pass
+        return
+        yield
 
     p = payload.AsyncIterablePayload(gen(), content_type="application/custom")
     assert p.content_type == "application/custom"
@@ -124,9 +126,9 @@ def test_async_iterable_payload_not_async_iterable() -> None:
 
 async def test_stream_reader_long_lines() -> None:
     loop = asyncio.get_event_loop()
-    DATA = b"0" * 1024 ** 3
+    DATA = b"0" * 1024**3
 
-    stream = streams.StreamReader(mock.Mock(), 2 ** 16, loop=loop)
+    stream = streams.StreamReader(mock.Mock(), 2**16, loop=loop)
     stream.feed_data(DATA)
     stream.feed_eof()
     body = payload.get_payload(stream)
