@@ -4,6 +4,7 @@ import socket
 import ssl
 import sys
 import unittest
+from typing import Any
 from unittest import mock
 
 import pytest
@@ -11,6 +12,7 @@ from yarl import URL
 
 import aiohttp
 from aiohttp.client_reqrep import ClientRequest, ClientResponse
+from aiohttp.connector import _SSL_CONTEXT_VERIFIED
 from aiohttp.helpers import TimerNoop
 from aiohttp.test_utils import make_mocked_coro
 
@@ -40,7 +42,12 @@ class TestProxy(unittest.TestCase):
         gc.collect()
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_connect(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_connect(self, start_connection: Any, ClientRequestMock: Any) -> None:
         req = ClientRequest(
             "GET",
             URL("http://www.python.org"),
@@ -54,7 +61,18 @@ class TestProxy(unittest.TestCase):
             return aiohttp.TCPConnector()
 
         connector = self.loop.run_until_complete(make_conn())
-        connector._resolve_host = make_mocked_coro([mock.MagicMock()])
+        connector._resolve_host = make_mocked_coro(
+            [
+                {
+                    "hostname": "hostname",
+                    "host": "127.0.0.1",
+                    "port": 80,
+                    "family": socket.AF_INET,
+                    "proto": 0,
+                    "flags": 0,
+                }
+            ]
+        )
 
         proto = mock.Mock(
             **{
@@ -75,13 +93,18 @@ class TestProxy(unittest.TestCase):
             auth=None,
             headers={"Host": "www.python.org"},
             loop=self.loop,
-            ssl=None,
+            ssl=True,
         )
 
         conn.close()
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_proxy_headers(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_proxy_headers(self, start_connection: Any, ClientRequestMock: Any) -> None:
         req = ClientRequest(
             "GET",
             URL("http://www.python.org"),
@@ -96,7 +119,18 @@ class TestProxy(unittest.TestCase):
             return aiohttp.TCPConnector()
 
         connector = self.loop.run_until_complete(make_conn())
-        connector._resolve_host = make_mocked_coro([mock.MagicMock()])
+        connector._resolve_host = make_mocked_coro(
+            [
+                {
+                    "hostname": "hostname",
+                    "host": "127.0.0.1",
+                    "port": 80,
+                    "family": socket.AF_INET,
+                    "proto": 0,
+                    "flags": 0,
+                }
+            ]
+        )
 
         proto = mock.Mock(
             **{
@@ -117,12 +151,17 @@ class TestProxy(unittest.TestCase):
             auth=None,
             headers={"Host": "www.python.org", "Foo": "Bar"},
             loop=self.loop,
-            ssl=None,
+            ssl=True,
         )
 
         conn.close()
 
-    def test_proxy_auth(self) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_proxy_auth(self, start_connection: Any) -> None:
         with self.assertRaises(ValueError) as ctx:
             ClientRequest(
                 "GET",
@@ -136,11 +175,16 @@ class TestProxy(unittest.TestCase):
             "proxy_auth must be None or BasicAuth() tuple",
         )
 
-    def test_proxy_dns_error(self) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_proxy_dns_error(self, start_connection: Any) -> None:
         async def make_conn():
             return aiohttp.TCPConnector()
 
-        connector = self.loop.run_until_complete(make_conn())
+        connector: aiohttp.TCPConnector = self.loop.run_until_complete(make_conn())
         connector._resolve_host = make_mocked_coro(
             raise_exception=OSError("dont take it serious")
         )
@@ -159,7 +203,12 @@ class TestProxy(unittest.TestCase):
         self.assertEqual(req.url.path, "/")
         self.assertEqual(dict(req.headers), expected_headers)
 
-    def test_proxy_connection_error(self) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_proxy_connection_error(self, start_connection: Any) -> None:
         async def make_conn():
             return aiohttp.TCPConnector()
 
@@ -192,7 +241,14 @@ class TestProxy(unittest.TestCase):
             )
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_proxy_server_hostname_default(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_proxy_server_hostname_default(
+        self, start_connection: Any, ClientRequestMock: Any
+    ) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -252,7 +308,14 @@ class TestProxy(unittest.TestCase):
         self.loop.run_until_complete(req.close())
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_proxy_server_hostname_override(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_proxy_server_hostname_override(
+        self, start_connection: Any, ClientRequestMock: Any
+    ) -> None:
         proxy_req = ClientRequest(
             "GET",
             URL("http://proxy.example.com"),
@@ -316,7 +379,14 @@ class TestProxy(unittest.TestCase):
         self.loop.run_until_complete(req.close())
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_https_connect(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_https_connect(
+        self, start_connection: mock.Mock, ClientRequestMock: mock.Mock
+    ) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -376,7 +446,14 @@ class TestProxy(unittest.TestCase):
         self.loop.run_until_complete(req.close())
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_https_connect_certificate_error(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_https_connect_certificate_error(
+        self, start_connection: Any, ClientRequestMock: Any
+    ) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -430,7 +507,14 @@ class TestProxy(unittest.TestCase):
             )
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_https_connect_ssl_error(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_https_connect_ssl_error(
+        self, start_connection: Any, ClientRequestMock: Any
+    ) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -486,7 +570,14 @@ class TestProxy(unittest.TestCase):
             )
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_https_connect_http_proxy_error(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_https_connect_http_proxy_error(
+        self, start_connection: Any, ClientRequestMock: Any
+    ) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -545,7 +636,14 @@ class TestProxy(unittest.TestCase):
         self.loop.run_until_complete(req.close())
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_https_connect_resp_start_error(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_https_connect_resp_start_error(
+        self, start_connection: Any, ClientRequestMock: Any
+    ) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -598,7 +696,12 @@ class TestProxy(unittest.TestCase):
             )
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_request_port(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_request_port(self, start_connection: Any, ClientRequestMock: Any) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -656,7 +759,14 @@ class TestProxy(unittest.TestCase):
         self.assertIsNone(req.proxy_auth)
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_https_connect_pass_ssl_context(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_https_connect_pass_ssl_context(
+        self, start_connection: Any, ClientRequestMock: Any
+    ) -> None:
         proxy_req = ClientRequest(
             "GET", URL("http://proxy.example.com"), loop=self.loop
         )
@@ -710,7 +820,7 @@ class TestProxy(unittest.TestCase):
         self.loop.start_tls.assert_called_with(
             mock.ANY,
             mock.ANY,
-            connector._make_ssl_context(True),
+            _SSL_CONTEXT_VERIFIED,
             server_hostname="www.python.org",
             ssl_handshake_timeout=mock.ANY,
         )
@@ -724,7 +834,12 @@ class TestProxy(unittest.TestCase):
         self.loop.run_until_complete(req.close())
 
     @mock.patch("aiohttp.connector.ClientRequest")
-    def test_https_auth(self, ClientRequestMock) -> None:
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    def test_https_auth(self, start_connection: Any, ClientRequestMock: Any) -> None:
         proxy_req = ClientRequest(
             "GET",
             URL("http://proxy.example.com"),
