@@ -1,10 +1,11 @@
 import asyncio
+import socket
 from contextlib import suppress
 from unittest import mock
 
 import pytest
 
-from aiohttp import client, helpers, web
+from aiohttp import client, web
 
 
 async def test_simple_server(aiohttp_raw_server, aiohttp_client) -> None:
@@ -19,12 +20,6 @@ async def test_simple_server(aiohttp_raw_server, aiohttp_client) -> None:
     assert txt == "/path/to"
 
 
-@pytest.mark.xfail(
-    not helpers.NO_EXTENSIONS,
-    raises=client.ServerDisconnectedError,
-    reason="The behavior of C-extensions differs from pure-Python: "
-    "https://github.com/aio-libs/aiohttp/issues/6446",
-)
 async def test_unsupported_upgrade(aiohttp_raw_server, aiohttp_client) -> None:
     # don't fail if a client probes for an unsupported protocol upgrade
     # https://github.com/aio-libs/aiohttp/issues/6446#issuecomment-999032039
@@ -175,9 +170,10 @@ async def test_raw_server_html_exception_debug(aiohttp_raw_server, aiohttp_clien
     logger.exception.assert_called_with("Error handling request", exc_info=exc)
 
 
-async def test_handler_cancellation(aiohttp_unused_port) -> None:
+async def test_handler_cancellation(unused_port_socket: socket.socket) -> None:
     event = asyncio.Event()
-    port = aiohttp_unused_port()
+    sock = unused_port_socket
+    port = sock.getsockname()[1]
 
     async def on_request(_: web.Request) -> web.Response:
         nonlocal event
@@ -195,10 +191,9 @@ async def test_handler_cancellation(aiohttp_unused_port) -> None:
     runner = web.AppRunner(app, handler_cancellation=True)
     await runner.setup()
 
-    site = web.TCPSite(runner, host="localhost", port=port)
+    site = web.SockSite(runner, sock=sock)
 
     await site.start()
-
     try:
         assert runner.server.handler_cancellation, "Flag was not propagated"
 
@@ -206,7 +201,7 @@ async def test_handler_cancellation(aiohttp_unused_port) -> None:
             timeout=client.ClientTimeout(total=0.1)
         ) as sess:
             with pytest.raises(asyncio.TimeoutError):
-                await sess.get(f"http://localhost:{port}/")
+                await sess.get(f"http://127.0.0.1:{port}/")
 
         with suppress(asyncio.TimeoutError):
             await asyncio.wait_for(event.wait(), timeout=1)
@@ -215,10 +210,11 @@ async def test_handler_cancellation(aiohttp_unused_port) -> None:
         await asyncio.gather(runner.shutdown(), site.stop())
 
 
-async def test_no_handler_cancellation(aiohttp_unused_port) -> None:
+async def test_no_handler_cancellation(unused_port_socket: socket.socket) -> None:
     timeout_event = asyncio.Event()
     done_event = asyncio.Event()
-    port = aiohttp_unused_port()
+    sock = unused_port_socket
+    port = sock.getsockname()[1]
     started = False
 
     async def on_request(_: web.Request) -> web.Response:
@@ -234,16 +230,15 @@ async def test_no_handler_cancellation(aiohttp_unused_port) -> None:
     runner = web.AppRunner(app)
     await runner.setup()
 
-    site = web.TCPSite(runner, host="localhost", port=port)
+    site = web.SockSite(runner, sock=sock)
 
     await site.start()
-
     try:
         async with client.ClientSession(
             timeout=client.ClientTimeout(total=0.2)
         ) as sess:
             with pytest.raises(asyncio.TimeoutError):
-                await sess.get(f"http://localhost:{port}/")
+                await sess.get(f"http://127.0.0.1:{port}/")
         await asyncio.sleep(0.1)
         timeout_event.set()
 
